#ifndef CKCoreCIFilters_h
#define CKCoreCIFilters_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

typedef enum eCIFILTERType: NSUInteger
  {
  CIFILTERTypeEffectMono = 0,
  CIFILTERTypeSepiaTone,
  CIFILTERTypePhotoEffectInstant,
  CIFILTERTypeVignetteEffect,
  CIFILTERTypePhotoEffectChrome,
  CIFILTERTypePhotoEffectNoir,
  CIFILTERTypePhotoEffectProcess,
  CIFILTERTypePhotoEffectTonal,
  CIFILTERTypePhotoEffectTransfer,
  CIFILTERTypeColorPosterize,
  CIFILTERTypeColorInvert,
  CIFILTERTypeVibrance,
  CIFILTERTypeExposure,
  CIFILTERTypeMedian,
  CIFILTERTypeBloom,
  CIFILTERComicEffect
  } eCIFILTERType;

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCIFilters: NSObject
  {
  CIContext                           *graphicContext;
  NSImage                             *colorMask;
  }
 
//-------------------------------------------------------------------------------------------------------------------------
+ (CKCoreCIFilters *)                  sharedInstance;

//-------------------------------------------------------------------------------------------------------------------------
- (NSImage *) scaleImage:(NSImage *) withImage toScale:(CGFloat) toScale;
- (NSImage *) scaleImage:(NSImage *) withImage toScale:(CGFloat) toScale toAspectRatio:(CGFloat) toAspectRatio;
- (NSImage *) cropImage:(NSImage *) withImage toRect:(CGRect) toRect;
- (NSImage *) gaussianBlurImage:(NSImage *) withImage;
- (NSImage *) gaussianBlurImage:(NSImage *) withImage withRadius:(CGFloat) withRadius;
- (NSImage *) transformImage:(NSImage *) withImage withTransform:(CGAffineTransform) withTransform;

- (NSImage *) sourceOverCompositingImage:(NSImage *) withBGImage withFGImage:(NSImage *) withFGImage toPoint:(CGPoint) toPoint;
- (NSImage *) photoEffectMonoImage:(NSImage *) withImage;
- (NSImage *) sepiaToneImage:(NSImage *) withImage withIntensity:(CGFloat) withIntensity;
- (NSImage *) photoEffectInstantImage:(NSImage *) withImage;
- (NSImage *) vignetteEffectInstantImage:(NSImage *) withImage withIntensity:(CGFloat) withIntensity;
- (NSImage *) photoEffectChromeImage:(NSImage *) withImage;
- (NSImage *) photoEffectNoirImage:(NSImage *) withImage;
- (NSImage *) photoEffectProcessImage:(NSImage *) withImage;
- (NSImage *) photoEffectTonalImage:(NSImage *) withImage;
- (NSImage *) photoEffectTransferImage:(NSImage *) withImage;
- (NSImage *) colorPosterizeImage:(NSImage *) withImage withLevels:(CGFloat) withLevels;
- (NSImage *) colorInvertImage:(NSImage *) withImage;
- (NSImage *) vibranceImage:(NSImage *) withImage withIntensity:(CGFloat) withIntensity;
- (NSImage *) exposureImage:(NSImage *) withImage withEV:(CGFloat) withEV;
- (NSImage *) medianImage:(NSImage *) withImage;
- (NSImage *) bloomImage:(NSImage *) withImage withIntensity:(CGFloat) withIntensity withRadius:(CGFloat) withRadius;
- (NSImage *) comicEffectImage:(NSImage *) withImage;

- (NSImage *) modTransitionImage:(NSImage *) fromImage toImage:(NSImage *) toImage withEffectStep:(CGFloat) withEffectStep;
- (NSImage *) dissolveTransitionImage:(NSImage *) fromImage toImage:(NSImage *) toImage withEffectStep:(CGFloat) withEffectStep;
- (NSImage *) colorToTransitionImage:(NSImage *) fromImage withEffectStep:(CGFloat) withEffectStep withColor:(NSColor *) withColor;
- (NSImage *) colorFromTransitionImage:(NSImage *) toImage withEffectStep:(CGFloat) withEffectStep withColor:(NSColor *) withColor;
- (NSImage *) whiteFromTransitionImage:(NSImage *) toImage withEffectStep:(CGFloat) withEffectStep;
- (NSImage *) whiteToTransitionImage:(NSImage *) fromImage withEffectStep:(CGFloat) withEffectStep;
- (NSImage *) blackToTransitionImage:(NSImage *) fromImage withEffectStep:(CGFloat) withEffectStep;
- (NSImage *) blackFromTransitionImage:(NSImage *) toImage withEffectStep:(CGFloat) withEffectStep;
- (NSImage *) deformBumpImage:(NSImage *) withImage withEffectStep:(CGFloat) withEffectStep;
- (NSImage *) flashTransitionImage:(NSImage *) fromImage toImage:(NSImage *) toImage withEffectStep:(CGFloat) withEffectStep;

- (void)      cleanUp;

@end

#endif

