#ifndef CKCoreCoinbase_h
#define CKCoreCoinbase_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

NS_ASSUME_NONNULL_BEGIN

//----------------------------------------------------------------------------------------------------
@interface CKCoreCoinbaseFees: NSObject

@property (nonatomic) CGFloat            totalFees;
@property (nonatomic) CGFloat            totalVolume;
@property (nonatomic) CGFloat            makerFeeRate;
@property (nonatomic) CGFloat            takerFeeRate;

- (id)      initWithData:(NSDictionary *) withData;

@end

//----------------------------------------------------------------------------------------------------
@interface CKCoreCoinbaseTrade: NSObject

@property (nonatomic) CGFloat            price;
@property (nonatomic) NSString          *productID;
@property (nonatomic) NSString          *side;
@property (nonatomic) CGFloat            size;
@property (nonatomic) NSDate            *tradeOn;
@property (nonatomic) NSString          *tradeID;
@property (nonatomic) CGFloat            bestAsk;
@property (nonatomic) CGFloat            bestBid;

- (id)      initWithData:(NSDictionary *) withData;
- (Boolean) isBuyOrder;
- (Boolean) isSellOrder;

@end

//----------------------------------------------------------------------------------------------------
@interface CKCoreCoinbaseFilledOrder: NSObject

@property (nonatomic) CGFloat            price;
@property (nonatomic) CGFloat            commission;
@property (nonatomic) NSString          *productID;
@property (nonatomic) NSString          *userID;
@property (nonatomic) NSString          *side;
@property (nonatomic) CGFloat            size;
@property (nonatomic) NSString          *tradeID;
@property (nonatomic) NSDate            *tradeOn;

- (id)      initWithData:(NSDictionary *) withData;
- (Boolean) isBuyOrder;
- (Boolean) isSellOrder;

@end

//----------------------------------------------------------------------------------------------------
@interface CKCoreCoinbaseProduct: NSObject

@property (nonatomic) CGFloat            price;
@property (nonatomic) CGFloat            priceChangePercentage24h;
@property (nonatomic) NSString          *productID;
@property (nonatomic) NSString          *productName;
@property (nonatomic) NSString          *productSymbol;
@property (nonatomic) CGFloat            volume24h;
@property (nonatomic) CGFloat            volumeChangePercentage24h;

- (id) initWithData:(NSDictionary *) withData;

@end

//----------------------------------------------------------------------------------------------------
@interface CKCoreCoinbaseAccount: NSObject

@property (nonatomic) Boolean            isActive;
@property (nonatomic) NSString          *balanceCurrency;
@property (nonatomic) CGFloat            balanceAmount;
@property (nonatomic) NSDate            *createdOn;
@property (nonatomic) NSDate            *deletedOn;
@property (nonatomic) NSString          *currency;
@property (nonatomic) Boolean            isDefault;
@property (nonatomic) NSString          *name;
@property (nonatomic) Boolean            isReady;
@property (nonatomic) NSString          *type;
@property (nonatomic) NSDate            *updatedOn;
@property (nonatomic) NSString          *uuid;

- (id) initWithData:(NSDictionary *) withData;

@end

//----------------------------------------------------------------------------------------------------
@interface CKCoreCoinbase: NSObject

@property (nonatomic) NSString               *apiKey;
@property (nonatomic) NSString               *apiSecret;

- (void) getPriceBySymbol:(NSString *) withSymbol completionBlock:(void (^)(CGFloat price, NSError *error)) completionBlock;
- (void) getProductTickerBySymbol:(NSString *) withSymbol withCount:(NSInteger) withCount completionBlock:(void (^)(NSArray <CKCoreCoinbaseTrade *> *listOfTrades, NSError *error)) completionBlock;
- (void) listProducts:(void (^)(NSArray <CKCoreCoinbaseProduct *> *listOfProducts, NSError *error)) completionBlock;
- (void) getProductBySymbol:(NSString *) withSymbol completionBlock:(void (^)(CKCoreCoinbaseProduct *result, NSError *error)) completionBlock;
- (void) listAccounts:(void (^)(NSArray <CKCoreCoinbaseAccount *> *listOfAccounts, NSError *error)) completionBlock;
- (void) getAccountInformation:(NSString *) withAccountID completionBlock:(void (^)(CKCoreCoinbaseAccount *account, NSError *error)) completionBlock;
- (void) listFilledOrders:(void (^)(NSArray <CKCoreCoinbaseFilledOrder *> *listOfOrders, NSError *error)) completionBlock;
- (void) listFilledOrders:(NSDictionary *) withCriterias completionBlock:(void (^)(NSArray <CKCoreCoinbaseFilledOrder *> *listOfOrders, NSError *error)) completionBlock;
- (void) listHistoricalOrders:(void (^)(NSDictionary *result, NSError *error)) completionBlock;
- (void) placeSellOrder:(NSString *) withSymbol withVolume:(CGFloat) withVolume completionBlock:(void (^)(NSDictionary *result, NSError *error)) completionBlock;
- (void) placeBuyOrder:(NSString *) withSymbol withAmount:(CGFloat) withAmount completionBlock:(void (^)(NSDictionary *result, NSError *error)) completionBlock;
- (void) cleanUp;
- (void) getTransactionSummary:(NSDate *) withStartDate withEndDate:(NSDate *) withEndDate withCurrency:(NSString *) withCurrency completionBlock:(void (^)(CKCoreCoinbaseFees *fees, NSError *error)) completionBlock;


- (void)fetchBTCPriceHistoryWithCompletion:(void (^)(NSDictionary * _Nullable result, NSError * _Nullable error))completion;

@end

//  CKCoreCoinbase *xx = [CKCoreCoinbase new];
//  
//  xx.apiKey    = @"xWXCRLlZiXYD1vfp";
//  xx.apiSecret = @"Q7weT3kUUEoyzJoswDSKXnV1ZYNzEyoT";
//  
//  [xx getProductBySymbol:@"LTC-USD" completionBlock:^(NSDictionary *result, NSError *error)
//    {
//    if (!error)
//      [xx placeSellOrder:@"LTC-USD" withVolume:[result floatForKeyname:@"base_min_size"] completionBlock:^(NSDictionary *result, NSError *error)
//        {
//        NSLog(@"");
//        }];
//    }];

NS_ASSUME_NONNULL_END

#endif

