#ifndef CKCoreCryptography_h
#define CKCoreCryptography_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

NS_ASSUME_NONNULL_BEGIN

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography: NSObject

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (RSA)

+ (Boolean)   RSAGenerateKeyPair:(NSString *) publicTagString privateTag:(NSString *) privateTagString;
+ (void)      RSAGenerateKeyPair:(NSString *) publicTagString privateTag:(NSString *) privateTagString completionBlock:(void (^)(NSString *pemPublicKey, NSString *pemPrivateKey)) completionBlock;
+ (SecKeyRef) RSALoadPublicKeyFromPEM:(NSString *) publicKey;
+ (SecKeyRef) RSALoadPrivateKeyFromPEM:(NSString *) privateKey;
+ (NSData *)  RSASignString:(NSString *) stringToSign privateKey:(SecKeyRef) privateKey;
+ (Boolean)   RSAVerifyStringSignature:(NSString *) stringToVerify signature:(NSData *) signature publicKey:(SecKeyRef) publicKey;
+ (SecKeyRef) RSALoadPublicKeyFromKeyChain:(NSString *) publicTagString;
+ (SecKeyRef) RSALoadPrivateKeyFromKeyChain:(NSString *) privateTagString;
+ (SecKeyRef) RSALoadPublicKeyFromPEMFile:(NSURL *) pemFileUrl;
+ (SecKeyRef) RSALoadPrivateKeyFromPEMFile:(NSURL *) pemFileUrl;
+ (Boolean)   RSAVerifyDataSignature:(NSData *) dataToVerify signature:(NSData *) signature publicKey:(SecKeyRef) publicKey;
+ (NSData *)  RSASignData:(NSData *) dataToSign privateKey:(SecKeyRef) privateKey;
+ (void)      RSAGenerateKeyPairFiles:(NSString *) withTag toPrivateKeyFilePath:(NSString *) toPrivateKeyFilePath toPublicKeyFilePath:(NSString *) toPublicKeyFilePath completionBlock:(void (^)(Boolean isSuccess)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (SHA256)

+ (NSString *_Nonnull) SHA256StringHash:(NSString *_Nonnull) withString;
+ (NSString *_Nonnull) SHA256DataHash:(NSData *_Nonnull) withData;
+ (NSString *_Nonnull) SHA256ForFileAtUrl:(NSURL *_Nonnull) withUrl;
+ (void)               SHA256ForFileAtUrl:(NSURL *) withUrl completionBlock:(void (^)(NSInteger exitCode, NSString *hashValue)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (SHA384)

+ (NSString *_Nonnull) SHA384StringHash:(NSString *_Nonnull) withString;
+ (NSString *_Nonnull) SHA384DataHash:(NSData *_Nonnull) withData;
+ (NSString *_Nonnull) SHA384ForFileAtUrl:(NSURL *_Nonnull) withUrl;
+ (void)               SHA384ForFileAtUrl:(NSURL *) withUrl completionBlock:(void (^)(NSInteger exitCode, NSString *hashValue)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (SHA512)

+ (NSString *_Nonnull) SHA512StringHash:(NSString *_Nonnull) withString;
+ (NSString *_Nonnull) SHA512DataHash:(NSData *_Nonnull) withData;
+ (NSString *_Nonnull) SHA512ForFileAtUrl:(NSURL *_Nonnull) withUrl;
+ (void)               SHA512ForFileAtUrl:(NSURL *) withUrl completionBlock:(void (^)(NSInteger exitCode, NSString *hashValue)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (SHA1)

+ (NSString *_Nonnull) SHA1StringHash:(NSString *_Nonnull) withString;
+ (NSString *_Nonnull) SHA1DataHash:(NSData *_Nonnull) withData;
+ (NSString *_Nonnull) SHA1ForFileAtUrl:(NSURL *_Nonnull) withUrl;
+ (void)               SHA1ForFileAtUrl:(NSURL *) withUrl completionBlock:(void (^)(NSInteger exitCode, NSString *hashValue)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (MD5)

+ (NSString *_Nonnull) MD5StringHash:(NSString *_Nonnull) withString;
+ (NSString *_Nonnull) MD5DataHash:(NSData *_Nonnull) withData;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (XOR)

+ (NSString *_Nonnull) XOREncrypt:(NSString *_Nonnull) valueIN;
+ (NSString *_Nonnull) XORDecrypt:(NSString *_Nonnull) valueIN;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (AES128)

+ (NSData *_Nullable) AES128Cipher:(NSData *_Nonnull) Key Value:(NSData *_Nonnull) Value Iv:(NSData *_Nonnull) Iv Operation:(unsigned int) Operation Options:(unsigned int) Options Output:(NSMutableData *_Nonnull) Output;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (HMAC256)

+ (NSString *) HMAC256HashToBase64:(NSString *) withString2Hash withKey:(NSString *) withKey;
+ (NSData *)   HMAC256HashToData:(NSString *) withString2Hash withKey:(NSString *) withKey;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreCryptography (AES256)

+ (NSData *_Nonnull)   AES256Encrypt:(NSData *_Nonnull) Data with256BitsKeyPlain:(NSString *_Nonnull) Key;
+ (NSData *_Nonnull)   AES256Decrypt:(NSData *_Nonnull) Data with256BitsKeyPlain:(NSString *_Nonnull) Key;
+ (NSString *_Nonnull) AES256EncryptString:(NSString *_Nonnull) Plaintext with256BitsKeyPlain:(NSString *_Nonnull) Key;
+ (NSString *_Nonnull) AES256DecryptString:(NSString *_Nonnull) Ciphertext with256BitsKeyPlain:(NSString *_Nonnull) Key;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
//@interface COCoreCryptography (RSA)
//
//+ (NSString *_Nullable) RSADecryptWithPublicKey:(NSString *_Nullable) withEncryptedBase64String withPEMPublicKeyResource:(NSString *_Nullable) withPEMPublicKeyResource;
//+ (NSString *_Nullable) RSAEncryptWithPrivateKey:(NSString *_Nullable) withString withPEMPrivateKeyResource:(NSString *_Nullable) withPEMPrivateKeyResource;
//+ (SecKeyRef _Nullable) RSAKeyRefOfKeyWithTag:(NSString *_Nullable) keyTag;
//+ (Boolean)             RSADeleteKeyWithTag:(NSString *_Nullable) keyTag;
//+ (Boolean)             RSAHasKeyWithTag:(NSString *_Nullable) keyTag;
//+ (NSData *_Nullable)   RSADataForKeyWithTag:(NSString *_Nullable) keyTag;
//+ (Boolean)             RSAGenerateKeyPairWithLength:(NSInteger) keyLength publicKeyTag:(NSString *_Nullable) publicKeyTag privateKeyTag:(NSString *_Nullable) privateKeyTag;
//
//@end

NS_ASSUME_NONNULL_END

#endif
