#ifndef CKCoreFileDownloader_h
#define CKCoreFileDownloader_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreFileDownloaderTask: NSObject

- (void) cleanUp;

@end

typedef void(^CKCoreFileDownloaderCompletionBlock)(NSURLResponse *response, NSURL *filePath, NSError *error, NSTimeInterval delay);
typedef void(^CKCoreFileDownloaderProgressBlock)(CKCoreFileDownloaderTask *task, CGFloat percentDone, CGFloat remainingTime);
typedef void(^CKCoreFileDownloaderCancelBlock)(void);

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreFileDownloader: NSObject <NSURLSessionDownloadDelegate>

- (void)                            cleanUp;
- (void)                            cancelDownload:(CKCoreFileDownloaderTask *) withTask;
- (void)                            cancelDownloads:(NSArray <CKCoreFileDownloaderTask *> *) withTasks;
- (void)                            clearAllDownloads;
- (void)                            clearDownload:(CKCoreFileDownloaderTask *) withTask;
- (void)                            clearDownloads:(NSArray <CKCoreFileDownloaderTask *> *) withTasks;
- (CKCoreFileDownloaderTask *)             downloadFile:(NSURL *) withSourceURL withTargetURL:(NSURL *) withTargetURL withCompletionBlock:(CKCoreFileDownloaderCompletionBlock) withCompletionBlock withProgressBlock:(CKCoreFileDownloaderProgressBlock) withProgressBlock withCancelBlock:(CKCoreFileDownloaderCancelBlock) withCancelBlock;
- (NSArray <CKCoreFileDownloaderTask *> *) downloadFiles:(NSArray<NSURL *> *) withSourceURLs withTargetURLs:(NSArray<NSURL *> *) withTargetURLs
                  withCompletionBlock:(void (^)(Boolean isSuccess, NSError *error, CGFloat requestDuration)) withCompletionBlock
                    withProgressBlock:(void (^)(NSInteger fileInProgress, CGFloat percentDone, CGFloat remainingTime)) withProgressBlock
                      withCancelBlock:(void (^)(void)) withCancelBlock;
- (NSArray <CKCoreFileDownloaderTask *> *) downloadFilesWithResponse:(NSArray<NSURL *> *) withSourceURLs withTargetURLs:(NSArray<NSURL *> *) withTargetURLs
   withCompletionBlock:(void (^)(Boolean isSuccess, NSHTTPURLResponse *response, NSError *error, CGFloat requestDuration)) withCompletionBlock
     withProgressBlock:(void (^)(NSInteger fileInProgress, CGFloat percentDone, CGFloat remainingTime)) withProgressBlock
       withCancelBlock:(void (^)(void)) withCancelBlock;

@end

#endif

