#ifndef CKCoreHighPrecisionTimer_h
#define CKCoreHighPrecisionTimer_h

#include <CoreFoundation/CoreFoundation.h>

typedef struct CKCoreHighPrecisionTimer_s *CKCoreHighPrecisionTimer;

typedef void (^CKCoreHighPrecisionTimer_Action)(void);

/**
 Creates a new instance of the timer, the time interval is in nano-seconds
 */
OSStatus CKCoreHighPrecisionTimer_Create(CKCoreHighPrecisionTimer* timer, uint64_t timeInterval);

/**
 set the time interval in nano-seconds
 */
OSStatus CKCoreHighPrecisionTimer_SetInterval(CKCoreHighPrecisionTimer timer, uint64_t timeInterval);

/**
 get the time interval in nano-seconds
 */
OSStatus CKCoreHighPrecisionTimer_GetInterval(const CKCoreHighPrecisionTimer timer, uint64_t* outTimeInterval);

/**
 The work amount is a real number ]0..1[ indicating the expected ratio of
 idle time and wait time, e.g. timer interval is 100ms, work is 1ms gives a
 work amount of 0.01.
 Must be set when timer is not running. See thread_time_constraint_policy_data_t
 and implementation for details.
 */
OSStatus CKCoreHighPrecisionTimer_SetWorkAmount(CKCoreHighPrecisionTimer timer, double workAmount);

/**
 Set the max number of timer repetitions
 */
OSStatus CKCoreHighPrecisionTimer_SetMaxRepetitions(CKCoreHighPrecisionTimer timer, uint64_t maxRepetitions);

/**
 Waits until the timer has stopped, only useful for timers with a repetition
 limit set
 */
OSStatus CKCoreHighPrecisionTimer_WaitUntilFinished(const CKCoreHighPrecisionTimer timer);

/**
 Start the timer to invoke its action repating with its interval.
 The action is performed on a real time thread therefor *must* finish fast.
 */
OSStatus CKCoreHighPrecisionTimer_Resume(CKCoreHighPrecisionTimer timer);

/**
 Suspend the execution of the timer
 */
OSStatus CKCoreHighPrecisionTimer_Suspend(CKCoreHighPrecisionTimer timer);

/**
 Dispose the timer
 */
OSStatus CKCoreHighPrecisionTimer_Dispose(CKCoreHighPrecisionTimer timer);

/*
 Set the block to be executed when the timer fires
 */
OSStatus CKCoreHighPrecisionTimer_SetAction(CKCoreHighPrecisionTimer timer, CKCoreHighPrecisionTimer_Action action);

#endif
