#ifndef CKCoreNSMenuItem_h
#define CKCoreNSMenuItem_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSMenuItem: NSMenuItem

@property (nonatomic) NSDictionary              *userData;

@end
 
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSMenuItem (Pretty)

// Parse a string into a menu item with a title and optional key equivalent.
//
// Examples:
//
//   [NSMenuItem nw_menuItemWithString:@"Preferences…  ⌘,"];
//   [NSMenuItem nw_menuItemWithString:@"Hide Others  ⌥⌘H"];
//   (note two spaces separating title from key equivalent);
//   [NSMenuItem nw_menuItemWithString:@"----------------"];
//
// Returns nil if parsing fails.
+ (NSMenuItem *) menuItemWithString:(NSString *)string;

// Same as above with an action too.
+ (NSMenuItem *) menuItemWithString:(NSString *) string action:(SEL) action;

// Same as above but may be an alternate menu item.
+ (NSMenuItem *) menuItemWithString:(NSString *) string action:(SEL) action isAlternate:(BOOL) isAlternate;

// Returns YES if the menu item string should parse as a separator.
extern BOOL IsPrettySeparator(NSString *item);

@end

#endif

