#ifndef CKCoreNSString_h
#define CKCoreNSString_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

NS_ASSUME_NONNULL_BEGIN

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSString

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSString (Extended)

- (Boolean)             hasPrefixInList:(NSArray <NSString *> *) withStringList;
- (Boolean)             hasSuffixInList:(NSArray <NSString *> *) withStringList;
- (Boolean)             isDifferentFromString:(NSString *) fromString;
- (NSString *_Nullable) trim;
- (NSString *_Nullable) trimLastCharacters:(NSUInteger) numberOfCharacters;
- (NSString *_Nullable) trimLastCharacter;
- (NSString *_Nullable) trimDisplayTextUpTo:(NSUInteger) numberOfCharacters;
- (NSString *_Nullable) stringAtIndex:(NSUInteger) index;
- (NSString *_Nullable) subStringFrom:(NSInteger) from to:(NSInteger) to;
- (NSString *_Nullable) firstCharacter;
- (NSString *_Nullable) lastCharacter;
- (BOOL)                containsTheCharacterFromSet:(NSCharacterSet *_Nullable) aSet options:(NSStringCompareOptions) mask;
- (void)                printString;
- (NSString *_Nullable) betweenStrings:(NSString *_Nullable) fromString toString:(NSString *_Nullable) toString;
- (NSString *_Nullable) firstNCharacters:(NSInteger) maxLength;
- (NSArray *_Nullable)  listOfCharacters;
- (Boolean)             isEqualToStringArray:(NSArray *) withArray;

+ (NSString *_Nullable) append:(id _Nullable) first, ...;
+ (CGFloat)             widthOfString:(NSString *_Nullable) string withFont:(NSFont *_Nullable) font;
+ (CGFloat)             heightOfString:(NSString *_Nullable) string withFont:(NSFont *_Nullable) font;
+ (NSString *_Nullable) removeNonAlphanumeric:(NSString *_Nullable) withString;
- (NSString *_Nullable) removeCharacters:(NSString *_Nullable) toRemove;
- (NSString *)          replaceCharactersBy:(NSString *) toRemove replaceBy:(NSString *) replaceBy;

- (NSString *_Nullable) toFilePath;
- (NSString *_Nullable) stripWhitespace;
- (NSString *_Nullable) removeAllWhitespaces;
- (Boolean)             isBlank;
- (NSInteger)           countOccurencesOfString:(NSString *) withString;
- (NSInteger)           wordsCount;
- (NSString *)          fileNameWithoutExtention;
- (NSString *)          fileNameWithExtention;
- (NSString *)          padWithSpaces:(NSInteger) toLength;
- (NSString *)          subStringAfter:(NSString *) withPattern withSubLength:(NSInteger) withSubLength;
- (NSInteger)           indexOfChar:(char) searchChar;
+ (NSString *)          capitalizedAlphabetLetterByIndex:(NSInteger) withIndex;
+ (NSString *)          alphabetLetterByIndex:(NSInteger) withIndex;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSString (URLEncode)

- (NSString *_Nullable) encodeURL;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSString (Sha1)

- (NSString *) Sha1;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSString (Download)

+ (void) downloadFromUrl:(NSURL *) withUrl completionBlock:(void (^)(NSString *string)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSString (Validation)

- (Boolean) isValidEmail;
- (Boolean) isNumeric;
- (Boolean) isEmpty;
- (Boolean) isValid;
- (Boolean) isCreditCardNumber;
- (Boolean) isValidBundleID;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSString (Convert)

+ (NSString *_Nullable) fromInteger:(NSInteger) fromValue;
+ (NSString *_Nullable) fromFloat:(CGFloat) fromValue;
+ (NSData *_Nullable)   fromHexString:(NSString *_Nullable) string;

- (NSString *_Nullable) toBASE64;
- (NSString *_Nullable) fromBASE16;

- (char *_Nullable)     toCharString;
- (NSArray *)           toCharactersArray;

@end

NS_ASSUME_NONNULL_END

#endif
