#ifndef CKCoreNSView_h
#define CKCoreNSView_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <QuartzCore/QuartzCore.h>

NS_ASSUME_NONNULL_BEGIN

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSView

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Color)

@property(nonatomic, retain) NSColor * _Nullable backColor;
@property(nonatomic, retain) NSColor * _Nullable borderColor;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Blur)

#if TARGET_OS_IOS
- (NSView *)  blurView:(UIBlurEffectStyle) withStyle;
- (UIImage *) blurredImage;
#endif

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
typedef NS_ENUM(NSUInteger, BCRectEdge)
  {
  /// The top edge of the rectangle.
  BCRectEdgeTop    = 0,
  /// The left edge of the rectangle.
  BCRectEdgeLeft   = 1,
  /// The bottom edge of the rectangle.
  BCRectEdgeBottom = 2,
  /// The right edge of the rectangle.
  BCRectEdgeRight  = 3
  };

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Genie)

#if TARGET_OS_IOS
- (void) genieInTransitionWithDuration:(NSTimeInterval) duration destinationRect:(CGRect) destRect destinationEdge:(BCRectEdge) destEdge completion:(void (^)(void)) completion;
- (void) genieOutTransitionWithDuration:(NSTimeInterval) duration startRect:(CGRect) startRect startEdge:(BCRectEdge) startEdge completion:(void (^)(void)) completion;
#endif

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Shimmer)

#if TARGET_OS_IOS
- (void) startShimmering;
- (void) stopShimmering;
#endif

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Frame)

@property (nonatomic, readonly) CGPoint boundCenter;
@property (nonatomic) CGPoint           centerXY;
@property (nonatomic, readonly) CGPoint centerInParentView;
@property (nonatomic, readonly) CGPoint centerInParentWindow;
@property (nonatomic) CGSize            size;
@property (nonatomic, readonly) CGSize  scaledSize;
@property (nonatomic) CGFloat           x;
@property (nonatomic) CGFloat           y;
@property (nonatomic) CGFloat           left;
@property (nonatomic) CGFloat           right;
@property (nonatomic) CGFloat           top;
@property (nonatomic) CGFloat           bottom;
@property (nonatomic) CGFloat           width;
@property (nonatomic, readonly) CGFloat halfWidth;
@property (nonatomic, readonly) CGFloat scaledWidth;
@property (nonatomic) CGFloat           height;
@property (nonatomic, readonly) CGFloat halfHeight;
@property (nonatomic, readonly) CGFloat scaledHeight;
@property (nonatomic) CGPoint           xy;

+ (void)         setViewsFrame:(NSArray *_Nonnull) withViews withFrame:(CGRect) withFrame;
- (void)         setWidth:(CGFloat) value keepAspectRatio:(Boolean) keepAspectRatio;
- (NSEdgeInsets) areaInsets;
- (CGFloat)      width:(CGFloat) withPercent;
- (CGFloat)      height:(CGFloat) withPercent;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Visibility)

@property (nonatomic) Boolean visible;

#if TARGET_OS_OSX
@property (nonatomic) CGFloat alpha;
#endif

+ (void) setViewsVisible:(NSArray *_Nonnull) withViews isVisible:(Boolean) isVisible;
+ (void) setViewsAlpha:(NSArray *) withViews withAlpha:(CGFloat) withAlpha;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Rounded)

- (void) makeItRounded:(CGFloat) cornerRadius withBorderColor:(nullable NSColor *) borderColor;
- (void) makeItRounded:(CGFloat) cornerRadius withBorderColor:(nullable NSColor *) borderColor withBackColor:(nullable NSColor *) backColor;
- (void) makeItRounded:(CGFloat) cornerRadius;
- (void) makeItRounded:(CGFloat) cornerRadius withBackColor:(NSColor *_Nonnull) backColor;
- (void) makeItRounded:(CGFloat) cornerRadius withBorderColor:(NSColor *_Nonnull) borderColor borderWidth:(CGFloat) borderWidth;
- (void) makeItRounded;
- (void) makeItRounded:(CGFloat) cornerRadius withBorderColor:(NSColor *) borderColor withBorderWidth:(CGFloat) borderWidth withBackColor:(NSColor *) backColor withShadowColor:(NSColor *) withShadowColor withShadowOffset:(CGSize) withShadowOffset withShadowRadius:(CGFloat) withShadowRadius;
- (void) makeItRounded:(CGFloat) cornerRadius withBorderColor:(NSColor *) borderColor borderWidth:(CGFloat) borderWidth withBackColor:(NSColor *) backColor;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Shadow)

- (void) dropShadow:(NSColor *_Nonnull) withColor withOffset:(CGSize) withOffset withBlurRadius:(CGFloat) withBlurRadius;
- (void) blurBorders:(CGFloat) withRadius;
- (void) dropShadow:(NSColor *) withColor withOffset:(CGSize) withOffset withBlurRadius:(CGFloat) withBlurRadius withCornersRadius:(CGFloat) withCornersRadius;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Snapshot)

+ (NSImage *) snapView:(NSView *) thisView;
+ (NSImage *) snapView:(NSView *) thisView afterUpdate:(Boolean) afterUpdate;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Rotation)

#if TARGET_OS_IOS
- (void) startRotating:(NSTimeInterval) withDuration withCount:(CGFloat) withRotationCount withKeyname:(NSString *) withKeyname;
- (void) stopRotating:(NSString *) withKeyname;
- (void) rotateByDegrees:(CGFloat) fromDegrees toDegrees:(CGFloat) toDegrees withDuration:(NSTimeInterval) withDuration;
#endif

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Shake)

- (void) shake;
- (void) shake:(int) times withDelta:(CGFloat) delta;
- (void) shake:(int) times withDelta:(CGFloat) delta completion:(nullable void (^)(void)) handler;
- (void) shake:(int) times withDelta:(CGFloat) delta speed:(NSTimeInterval) interval;
- (void) shake:(int) times withDelta:(CGFloat) delta speed:(NSTimeInterval) interval completion:(nullable void (^)(void)) handler;
- (void) shake:(int) times withDelta:(CGFloat) delta speed:(NSTimeInterval) interval shakeDirection:(NSInteger) shakeDirection;
- (void) shake:(int) times withDelta:(CGFloat) delta speed:(NSTimeInterval) interval shakeDirection:(NSInteger) shakeDirection completion:(nullable void (^)(void)) handler;
- (void) shake:(int) times direction:(int) direction currentTimes:(int) current withDelta:(CGFloat) delta speed:(NSTimeInterval) interval shakeDirection:(NSInteger) shakeDirection completion:(nullable void (^)(void)) handler;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
typedef NS_ENUM(NSUInteger, eNSViewAlignment)
  {
  eNSViewAlignmentLeft,
  eNSViewAlignmentCenterLeft,
  eNSViewAlignmentCenter,
  eNSViewAlignmentRight,
  eNSViewAlignmentCenterRight,
  eNSViewAlignmentTop,
  eNSViewAlignmentBottom
  };

@interface NSView (ViewPlacement)

- (void) placeSubviewAtMyLocation:(NSView *) thisSubview withLocation:(CGPoint) withLocation;
- (void) placeSubviewAtMyLocation:(NSView *) thisSubview withPercentLocation:(CGPoint) withPercentLocation;
- (void) placeSubviewAtMyCenterLocation:(NSView *) thisSubview withLocation:(CGPoint) withLocation;
- (void) placeSubviewAtMyCenterLocation:(NSView *) thisSubview withPercentLocation:(CGPoint) withPercentLocation;

- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withOffsetX:(CGFloat) withOffsetX;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withOffsetXY:(CGFloat) withOffsetXY;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withPercentOffset:(CGSize) withPercentOffset;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withPercentOffsetX:(CGFloat) withPercentOffsetX;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withPercentOffsetY:(CGFloat) withPercentOffsetY;
- (void) placeSubviewAtMyCenter:(NSView *) thisSubview withPercentOffsetXY:(CGFloat) withPercentOffsetXY;
- (void) placeSubviewsAtMyCenter:(NSArray<NSView *> *) thisSubviews;

- (void) placeSubviewAtMyCenterHorizontal:(NSView *) thisSubview;
- (void) placeSubviewAtMyCenterHorizontal:(NSView *) thisSubview withPercentOffsetX:(CGFloat) withPercentOffsetX;
- (void) placeSubviewAtMyCenterHorizontal:(NSView *) thisSubview withOffsetX:(CGFloat) withOffsetX;

- (void) placeSubviewAtMyCenterTop:(NSView *) thisSubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubviewAtMyCenterTop:(NSView *) thisSubview;
- (void) placeSubviewAtMyCenterTop:(NSView *) thisSubview withPercentOffsetY:(CGFloat) withPercentOffsetY;

- (void) placeSubviewAtMyCenterVertical:(NSView *) thisSubview;
- (void) placeSubviewAtMyCenterVertical:(NSView *) thisSubview withPercentOffsetY:(CGFloat) withPercentOffsetY;
- (void) placeSubviewAtMyCenterVertical:(NSView *) thisSubview withOffsetY:(CGFloat) withOffsetY;

- (void) placeSubviewAtMyRight:(NSView *) thisSubview withOffsetX:(CGFloat) withOffsetX;
- (void) placeSubviewAtMyRight:(NSView *) thisSubview;
- (void) placeSubviewAtMyRight:(NSView *) thisSubview withPercentOffsetX:(CGFloat) withPercentOffsetX;
- (void) placeSubviewAtMyCenterRight:(NSView *) thisSubview withOffsetX:(CGFloat) withOffsetX;
- (void) placeSubviewAtMyCenterRight:(NSView *) thisSubview withPercentOffsetX:(CGFloat) withPercentOffsetX;

- (void) placeSubviewAtMyLeft:(NSView *) thisSubview withOffsetX:(CGFloat) withOffsetX;
- (void) placeSubviewAtMyLeft:(NSView *) thisSubview;
- (void) placeSubviewAtMyLeft:(NSView *) thisSubview withPercentOffsetX:(CGFloat) withPercentOffsetX;
- (void) placeSubviewAtMyCenterLeft:(NSView *) thisSubview withOffsetX:(CGFloat) withOffsetX;
- (void) placeSubviewAtMyCenterLeft:(NSView *) thisSubview withPercentOffsetX:(CGFloat) withPercentOffsetX;

- (void) placeSubviewAtMyBottom:(NSView *) thisSubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubviewAtMyBottom:(NSView *) thisSubview;
- (void) placeSubviewAtMyBottom:(NSView *) thisSubview withPercentOffsetY:(CGFloat) withPercentOffsetY;

- (void) placeSubviewAtMyCenterBottom:(NSView *) thisSubview;
- (void) placeSubviewAtMyCenterBottom:(NSView *) thisSubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubviewAtMyCenterBottom:(NSView *) thisSubview withPercentOffsetY:(CGFloat) withPercentOffsetY;

- (void) placeSubviewAtMyTop:(NSView *) thisSubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubviewAtMyTop:(NSView *) thisSubview;
- (void) placeSubviewAtMyTop:(NSView *) thisSubview withPercentOffsetY:(CGFloat) withPercentOffsetY;

- (void) placeSubviewAtMyTopLeftCorner:(NSView *) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtMyTopLeftCorner:(NSView *) thisSubview withPercentOffset:(CGSize) withPercentOffset;
- (void) placeSubviewAtMyTopLeftCorner:(NSView *) thisSubview;
- (void) placeSubviewAtMyTopLeftCorner:(NSView *) thisSubview withOffsetXY:(CGFloat) withOffsetXY;
- (void) placeSubviewAtMyTopLeftCorner:(NSView *) thisSubview withPercentOffsetXY:(CGFloat) withPercentOffsetXY;

- (void) placeSubviewAtMyTopRightCorner:(NSView *) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtMyTopRightCorner:(NSView *) thisSubview withPercentOffset:(CGSize) withPercentOffset;
- (void) placeSubviewAtMyTopRightCorner:(NSView *) thisSubview;
- (void) placeSubviewAtMyTopRightCorner:(NSView *) thisSubview withOffsetXY:(CGFloat) withOffsetXY;
- (void) placeSubviewAtMyTopRightCorner:(NSView *) thisSubview withPercentOffsetXY:(CGFloat) withPercentOffsetXY;

- (void) placeSubviewAtMyBottomLeftCorner:(NSView *) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtMyBottomLeftCorner:(NSView *) thisSubview withOffsetXY:(CGFloat) withOffsetXY;
- (void) placeSubviewAtMyBottomLeftCorner:(NSView *) thisSubview withPercentOffsetXY:(CGFloat) withPercentOffsetXY;
- (void) placeSubviewAtMyBottomLeftCorner:(NSView *) thisSubview withPercentOffset:(CGSize) withPercentOffset;
- (void) placeSubviewAtMyBottomLeftCorner:(NSView *) thisSubview;

- (void) placeSubviewAtMyBottomRightCorner:(NSView *) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtMyBottomRightCorner:(NSView *) thisSubview withPercentOffset:(CGSize) withPercentOffset;
- (void) placeSubviewAtMyBottomRightCorner:(NSView *) thisSubview;
- (void) placeSubviewAtMyBottomRightCorner:(NSView *) thisSubview withOffsetXY:(CGFloat) withOffsetXY;
- (void) placeSubviewAtMyBottomRightCorner:(NSView *) thisSubview withPercentOffsetXY:(CGFloat) withPercentOffsetXY;

- (void) placeSubview:(NSView *) thisSubview leftOfMySubview:(NSView *) leftOfMySubview withOffsetX:(CGFloat) withOffsetX;
- (void) placeSubview:(NSView *) thisSubview leftOfMySubview:(NSView *) leftOfMySubview withPercentOffsetX:(CGFloat) withPercentOffsetX;
- (void) placeSubview:(NSView *) thisSubview leftOfMySubview:(NSView *) leftOfMySubview;
- (void) placeSubview:(NSView *) thisSubview leftOfMySubview:(NSView *) leftOfMySubview withOffsetX:(CGFloat) withOffsetX withAlignment:(eNSViewAlignment) withAlignment;

- (void) placeSubview:(NSView *) thisSubview rightOfMySubview:(NSView *) rightOfMySubview withOffsetX:(CGFloat) withOffsetX;
- (void) placeSubview:(NSView *) thisSubview rightOfMySubview:(NSView *) rightOfMySubview withPercentOffsetX:(CGFloat) withPercentOffsetX;
- (void) placeSubview:(NSView *) thisSubview rightOfMySubview:(NSView *) rightOfMySubview;

- (void) placeSubview:(NSView *) thisSubview topOfMySubview:(NSView *) topOfMySubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubview:(NSView *) thisSubview topOfMySubview:(NSView *) topOfMySubview withPercentOffsetY:(CGFloat) withPercentOffsetY;
- (void) placeSubview:(NSView *) thisSubview topOfMySubview:(NSView *) topOfMySubview;

- (void) placeSubview:(NSView *) thisSubview bottomOfMySubview:(NSView *) bottomOfMySubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubview:(NSView *) thisSubview bottomOfMySubview:(NSView *) bottomOfMySubview withPercentOffsetY:(CGFloat) withPercentOffsetY;
- (void) placeSubview:(NSView *) thisSubview bottomOfMySubview:(NSView *) bottomOfMySubview;

- (void) placeSubview:(NSView *) thisSubview centerOfMySubview:(NSView *) centerOfMySubview withOffsetY:(CGFloat) withOffsetY;
- (void) placeSubview:(NSView *) thisSubview centerOfMySubview:(NSView *) centerOfMySubview withPercentOffsetY:(CGFloat) withPercentOffsetY;
- (void) placeSubview:(NSView *) thisSubview centerOfMySubview:(NSView *) centerOfMySubview;

- (void) placeSubviews:(NSArray <NSView *> *) thisSubviews atMyLocations:(NSArray <NSValue *> *) atMyLocations;
- (void) placeSubviews:(NSArray <NSView *> *) thisSubviews atMyPercentLocations:(NSArray <NSValue *> *) atMyPercentLocations;
- (void) placeSubviews:(NSArray <NSView *> *) thisSubviews atMyCenterLocations:(NSArray <NSValue *> *) atMyCenterLocations;
- (void) placeSubviews:(NSArray <NSView *> *) thisSubviews atMyPercentCenterLocations:(NSArray <NSValue *> *) atMyPercentCenterLocations;

- (void) distributeInACirclePath:(NSArray *) thisSubviews withMyCenterPoint:(CGPoint) withMyCenterPoint withRadius:(CGFloat) withRadius;
- (void) distributeInACirclePath:(NSArray *) thisSubviews withMyPercentCenterPoint:(CGPoint) withMyPercentCenterPoint withRadius:(CGFloat) withRadius;

- (void) stackSubviews:(NSArray <NSView *> *) thisSubviews withAlignment:(eNSViewAlignment) withAlignment withSpacing:(CGFloat) withSpacing atMyLocation:(CGPoint) atMyLocation;
- (void) stackSubviews:(NSArray <NSView *> *) thisSubviews withAlignment:(eNSViewAlignment) withAlignment withPercentSpacing:(CGFloat) withPercentSpacing atMyPercentLocation:(CGPoint) atMyPercentLocation;















- (void) placeSubviewAtCenter:(NSView *_Nullable) thisSubview;
- (void) placeSubviewsAtCenter:(NSArray <NSView *> *) thisSubviews;
- (void) placeSubviewAtCenter:(NSView *_Nullable) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtCenterHorizontal:(NSView *_Nullable) thisSubview;
- (void) placeSubviewsAtCenterHorizontal:(NSArray *_Nullable) thisSubviews;
- (void) placeSubviewAtCenterVertical:(NSView *_Nullable) thisSubview;
- (void) placeSubviewAtCenterLeft:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewAtCenterTop:(NSView *_Nullable) thisSubview;
- (void) placeSubviewAtCenterTop:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewAtCenterBottom:(NSView *_Nullable) thisSubview;
- (void) placeSubviewsAtCenterBottom:(NSArray<NSView *> *) thisSubviews;
- (void) placeSubviewAtCenterBottom:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewsAtCenterBottom:(NSArray<NSView *> *) thisSubviews withOffset:(CGFloat) withOffset;
- (void) placeSubviewAtCenterBottom:(NSView *) thisSubview aboveThisSubview:(NSView *) aboveThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewAtCenterRight:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;

- (void) placeSubviewAtTop:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewAtTop:(NSView *_Nullable) thisSubview;
- (void) placeSubviewAtTopRightCorner:(NSView *_Nullable) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtTopRightCorner:(NSView *_Nullable) thisSubview withXYOffset:(CGFloat) withOffset;
- (void) placeSubviewAtTopLeftCorner:(NSView *_Nullable) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtTopLeftCorner:(NSView *_Nullable) thisSubview withXYOffset:(CGFloat) withOffset;

- (void) placeSubviewAtBottom:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewAtBottom:(NSView *_Nullable) thisSubview;
- (void) placeSubviewAtBottomLeftCorner:(NSView *_Nullable) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtBottomLeftCorner:(NSView *_Nullable) thisSubview withXYOffset:(CGFloat) withOffset;
- (void) placeSubviewAtBottomRightCorner:(NSView *_Nullable) thisSubview withOffset:(CGSize) withOffset;
- (void) placeSubviewAtBottomRightCorner:(NSView *_Nullable) thisSubview withXYOffset:(CGFloat) withOffset;

- (void) placeSubviewAtRight:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewAtLeft:(NSView *_Nullable) thisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *_Nullable) thisSubview belowThisSubview:(NSView *_Nullable) belowThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *_Nullable) thisSubview aboveThisSubview:(NSView *_Nullable) aboveThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *_Nullable) thisSubview leftOfThisSubview:(NSView *_Nullable) leftOfThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *_Nullable) thisSubview rightOfThisSubview:(NSView *_Nullable) rightOfThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubviewsAtCenterVertical:(NSArray *_Nullable) thisSubviews;
- (void) placeSubviewAtVerticalBetween:(NSView *_Nullable) thisSubview withSubviews:(NSArray *_Nullable) withSubviews;
- (void) placeSubviewsAtCenter:(NSArray *_Nullable) thisSubviews withYSpacing:(CGFloat) withYSpacing;
- (void) placeSubview:(NSView *) thisSubview centeredBelowThisSubview:(NSView *) belowThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *) thisSubview centeredAboveThisSubview:(NSView *) aboveThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *) thisSubview centeredRightOfThisSubview:(NSView *) rightOfThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *) thisSubview centeredLeftOfThisSubview:(NSView *) leftOfThisSubview withOffset:(CGFloat) withOffset;
- (void) placeSubview:(NSView *) thisSubview centeredToThisSubview:(NSView *) centeredToThisSubview withOffset:(CGSize) withOffset;

- (CGFloat) stackSubviewsHorizontallyToRight:(NSArray *) thisSubviews withOffset:(CGFloat) withOffset;
- (CGFloat) stackSubviewsHorizontallyToRight:(NSArray *) thisSubviews withOffset:(CGFloat) withOffset visibleOnly:(Boolean) visibleOnly;

- (void)    distributeInACirclePath:(NSArray *) thisSubviews withCenterPoint:(CGPoint) withCenterPoint withRadius:(CGFloat) withRadius;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (LayerPlacement)

- (void) placeLayerAtCenter:(CALayer *_Nullable) thisLayer;
- (void) placeLayerAtCenter:(CALayer *_Nullable) thisLayer withOffset:(CGSize) withOffset;
- (void) placeLayerAtCenterTop:(CALayer *_Nullable) thisLayer;
- (void) placeLayerAtCenterTop:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtTop:(CALayer *_Nullable) thisLayer;
- (void) placeLayerAtTop:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtBottom:(CALayer *_Nullable) thisLayer;
- (void) placeLayerAtBottom:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtCenterRight:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtCenterLeft:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtCenterBottom:(CALayer *_Nullable) thisLayer;
- (void) placeLayerAtCenterBottom:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtRight:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtLeft:(CALayer *_Nullable) thisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtCenterHorizontal:(CALayer *_Nullable) thisLayer;
- (void) placeLayerAtCenterVertical:(CALayer *_Nullable) thisLayer;
- (void) placeLayersAtCenterVertical:(NSArray *_Nullable) thisLayers;
- (void) placeLayer:(CALayer *_Nullable) thisLayer belowThisLayer:(CALayer *_Nullable) belowThisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayer:(CALayer *_Nullable) thisLayer aboveThisLayer:(CALayer *_Nullable) aboveThisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayer:(CALayer *_Nullable) thisLayer leftOfThisLayer:(CALayer *_Nullable) leftOfThisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayer:(CALayer *_Nullable) thisLayer rightOfThisLayer:(CALayer *_Nullable) rightOfThisLayer withOffset:(CGFloat) withOffset;
- (void) placeLayerAtTopRightCorner:(CALayer *_Nullable) thisLayer withOffset:(CGSize) withOffset;
- (void) placeLayerAtBottomRightCorner:(CALayer *_Nullable) thisLayer withOffset:(CGSize) withOffset;
- (void) placeLayerAtTopLeftCorner:(CALayer *_Nullable) thisLayer withOffset:(CGSize) withOffset;
- (void) placeLayerAtBottomLeftCorner:(CALayer *_Nullable) thisLayer withOffset:(CGSize) withOffset;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Subviews)

- (void)                 deleteAllSubviews;
- (void)                 addSubviews:(NSArray *_Nullable) withViews;
- (void)                 addSubviews:(NSArray *_Nullable) withViews underView:(id _Nullable) underView;
- (NSView *)             topSuperview;
- (void)                 bringToFront;

#if TARGET_OS_OSX
- (NSWindowController *) windowController;
#endif

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Scale)

- (void)    scaleToPercentSize:(CGFloat) withPercent;
- (void)    scaleToSize:(CGSize) withSize;
- (void)    scaleToHeight:(CGFloat) withHeight;
- (void)    scaleToWidth:(CGFloat) withWidth;
- (void)    scaleToFitSize:(CGSize) withSize;
- (void)    scaleToFillSize:(CGSize) withSize;
- (Boolean) isLandscape;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Glow)

- (void) dropGlow:(NSColor *) withColor withBlurRadius:(CGFloat) withBlurRadius;

#if TARGET_OS_IOS
@property (nonatomic, readonly) NSView *_Nullable glowView;

// Fade up, then down.
- (void) glowOnce;

// Useful for indicating "this object should be over there"
- (void) glowOnceAtLocation:(CGPoint) point inView:(NSView *_Nullable) view;
- (void) startGlowing;
- (void) startGlowingWithColor:(NSColor *_Nullable) color intensity:(CGFloat) intensity;
- (void) stopGlowing;
#endif

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Explode) <CAAnimationDelegate>

- (void) explodeWithGravity;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Borders)

- (CALayer *) createTopBorderWithHeight: (CGFloat) height andColor:(NSColor *)color;
- (NSView *)  createViewBackedTopBorderWithHeight: (CGFloat) height andColor:(NSColor *)color;
- (void)      addTopBorderWithHeight:(CGFloat) height andColor:(NSColor *)color;
- (void)      addViewBackedTopBorderWithHeight:(CGFloat) height andColor:(NSColor *)color;
- (CALayer *) createTopBorderWithHeight: (CGFloat) height color:(NSColor *)color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andTopOffset:(CGFloat)topOffset;
- (NSView *)  createViewBackedTopBorderWithHeight: (CGFloat) height color:(NSColor *)color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andTopOffset:(CGFloat)topOffset;
- (void)      addTopBorderWithHeight: (CGFloat) height color:(NSColor *)color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andTopOffset:(CGFloat)topOffset;
- (void)      addViewBackedTopBorderWithHeight: (CGFloat) height color:(NSColor *)color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andTopOffset:(CGFloat)topOffset;
- (CALayer *) createRightBorderWithWidth: (CGFloat) width andColor:(NSColor *) color;
- (NSView *)  createViewBackedRightBorderWithWidth: (CGFloat) width andColor:(NSColor *) color;
- (void)      addRightBorderWithWidth: (CGFloat) width andColor:(NSColor *) color;
- (void)      addViewBackedRightBorderWithWidth: (CGFloat) width andColor:(NSColor *) color;
- (CALayer *) createRightBorderWithWidth: (CGFloat) width color:(NSColor *) color rightOffset:(CGFloat)rightOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;
- (NSView *)  createViewBackedRightBorderWithWidth: (CGFloat) width color:(NSColor *) color rightOffset:(CGFloat)rightOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;
- (void)      addRightBorderWithWidth: (CGFloat) width color:(NSColor *) color rightOffset:(CGFloat)rightOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;
- (void)      addViewBackedRightBorderWithWidth: (CGFloat) width color:(NSColor *) color rightOffset:(CGFloat)rightOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;
- (CALayer *) createBottomBorderWithHeight: (CGFloat) height andColor:(NSColor *) color;
- (NSView *)  createViewBackedBottomBorderWithHeight: (CGFloat) height andColor:(NSColor *) color;
- (void)      addBottomBorderWithHeight:(CGFloat) height andColor:(NSColor *) color;
- (void)      addViewBackedBottomBorderWithHeight:(CGFloat) height andColor:(NSColor *) color;
- (CALayer *) createBottomBorderWithHeight: (CGFloat) height color:(NSColor *) color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andBottomOffset:(CGFloat)bottomOffset;
- (NSView *)  createViewBackedBottomBorderWithHeight: (CGFloat) height color:(NSColor *) color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andBottomOffset:(CGFloat)bottomOffset;
- (void)      addBottomBorderWithHeight: (CGFloat) height color:(NSColor*) color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andBottomOffset:(CGFloat)bottomOffset;
- (void)      addViewBackedBottomBorderWithHeight: (CGFloat) height color:(NSColor *) color leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset andBottomOffset:(CGFloat)bottomOffset;
- (CALayer *) createLeftBorderWithWidth: (CGFloat) width andColor:(NSColor *) color;
- (NSView *)  createViewBackedLeftBorderWithWidth: (CGFloat)width andColor:(NSColor *) color;
- (void)      addLeftBorderWithWidth: (CGFloat) width andColor:(NSColor *) color;
- (void)      addViewBackedLeftBorderWithWidth: (CGFloat) width andColor:(NSColor *) color;
- (CALayer *) createLeftBorderWithWidth: (CGFloat) width color:(NSColor *) color leftOffset:(CGFloat)leftOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;
- (NSView *)  createViewBackedLeftBorderWithWidth: (CGFloat) width color:(NSColor *) color leftOffset:(CGFloat)leftOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;
- (void)      addLeftBorderWithWidth: (CGFloat) width color:(NSColor *) color leftOffset:(CGFloat)leftOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;
- (void)      addViewBackedLeftBorderWithWidth: (CGFloat) width color:(NSColor *) color leftOffset:(CGFloat)leftOffset topOffset:(CGFloat)topOffset andBottomOffset:(CGFloat)bottomOffset;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSView (Animate)

#if TARGET_OS_OSX
- (void) animateMoveToView:(NSView *) mainView toView:(NSView *) toView borderColor:(NSColor *) borderColor;
- (void) animateMoveToRect:(NSView *) mainView toRect:(CGRect) toRect borderColor:(NSColor *) borderColor;
- (void) animateMoveToRect:(NSView *) mainView toRect:(CGRect) toRect borderColor:(NSColor *) borderColor image:(NSImage *) image;
- (void) animateRotate:(NSTimeInterval) withDelay;
- (void) animateScaleTo:(CGSize) toSize withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
#endif

- (void) animateMoveToRect:(CGRect) toRect withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateMoveVertically:(CGFloat) toY withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateMoveHorizontally:(CGFloat) toX withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateMoveTo:(CGPoint) toXY withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateMoveScaleTo:(CGPoint) toXY toSize:(CGSize) toSize withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateFadeOut:(CGFloat) withDuration completionBlock:(void (^ __nullable)(void)) completionBlock;
- (void) animateFadeOutWithDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateFadeIn:(CGFloat) withDuration completionBlock:(void (^ __nullable)(void)) completionBlock;
- (void) animateFadeInWithDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateFadeOutWithDuration:(NSTimeInterval) withDuration withFinalAlpha:(CGFloat) withFinalAlpha completionBlock:(void (^)(void)) completionBlock;
- (void) animateFadeInWithDuration:(NSTimeInterval) withDuration withFinalAlpha:(CGFloat) withFinalAlpha completionBlock:(void (^)(void)) completionBlock;
- (void) animateHighlight:(void (^ __nullable)(void)) completionBlock;
- (void) animateHighlight:(CGFloat) expandFactor completionBlock:(void (^ __nullable)(void)) completionBlock;
- (void) animatePopHighlight:(void (^ __nullable)(void)) completionBlock;
- (void) animateMoveTo:(CGPoint) toXY withAlpha:(CGFloat) withAlpha withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;
- (void) animateHighlight:(CGFloat) expandFactor duration:(NSTimeInterval) duration completionBlock:(void (^ __nullable)(void)) completionBlock;
- (void) animateFadeOutWithDuration:(NSTimeInterval) withDuration withFinalAlpha:(CGFloat) withFinalAlpha withMoveTo:(CGPoint) withMoveTo completionBlock:(void (^)(void)) completionBlock;
- (void) animateWidthTo:(CGFloat) toWidth withDuration:(NSTimeInterval) withDuration completionBlock:(void (^)(void)) completionBlock;

@end

NS_ASSUME_NONNULL_END

#endif

