#ifndef CKCoreNotifications_h
#define CKCoreNotifications_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

NS_ASSUME_NONNULL_BEGIN

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreLocalNotifications: NSObject

- (void) requestPermission:(void (^)(Boolean isGranted, NSError *error)) completionHandler;

+ (void) scheduleNotification:(NSString *) withTitle withBody:(NSString *) withBody fireAt:(NSDate *) fireAt completionHandler:(void (^)(NSError *error)) completionHandler;
+ (void) clearAllDeliveredNotifications;
+ (void) clearAllPendingNotifications;
+ (void) clearPendingNotificationsByID:(NSArray *) listOfIDs;
+ (void) clearDeliveredNotificationsByID:(NSArray *) listOfIDs;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreFSNotifications: NSObject

@property (nonatomic) id _Nullable          delegate;
@property (nonatomic) SEL _Nullable         onFoldersChanged;

- (void) cleanUp;
- (void) watchFolders:(NSArray *_Nullable) filesToWatch;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNotifications: NSObject

- (void) cleanUp;
- (void) addObserver:(NSNotificationName _Nullable) forName object:(nullable id) withObject queue:(nullable NSOperationQueue *) withQueue usingBlock:(void (^_Nullable)(NSNotification * _Nullable note)) withBlock;
- (void) addObserver:(NSNotificationName _Nullable) forName usingBlock:(void (^ _Nullable)(NSNotification * _Nullable note)) withBlock;
- (void) addObservers:(NSArray <NSNotificationName> *_Nonnull) forNames object:(nullable id) withObject queue:(nullable NSOperationQueue *) withQueue usingBlock:(void (^_Nullable)(NSNotification * _Nullable note)) withBlock;
- (void) addObservers:(NSArray <NSNotificationName> *_Nonnull) forNames usingBlock:(void (^ _Nullable)(NSNotification * _Nullable note)) withBlock;

+ (void) addObserver:(NSNotificationName) forName object:(nullable id) withObject queue:(nullable NSOperationQueue *) withQueue usingBlock:(void (^)(NSNotification *note)) withBlock;
+ (void) removeObserver:(id _Nonnull) withObserverID;
+ (void) post:(NSNotificationName _Nonnull) toName;
+ (void) post:(NSNotificationName _Nullable) toName afterDelay:(CGFloat) afterDelay;
+ (void) post:(NSNotificationName _Nonnull) toName object:(nullable id) withObject;
+ (void) post:(NSNotificationName _Nullable) toName object:(nullable id) withObject afterDelay:(CGFloat) afterDelay;
+ (void) post:(NSNotificationName _Nonnull) toName object:(nullable id) withObject userInfo:(nullable NSDictionary *) withUserInfo;
+ (void) post:(NSNotificationName _Nullable) toName object:(nullable id) withObject userInfo:(nullable NSDictionary *) withUserInfo afterDelay:(CGFloat) afterDelay;
+ (void) post:(NSNotificationName) toName userInfo:(nullable NSDictionary *) withUserInfo;

@end

NS_ASSUME_NONNULL_END

#endif
