#ifndef CKCoreNSColor_h
#define CKCoreNSColor_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #import <CodKits_macOS/CodKits_macOS.h>
#else
  #import <CodKits_iOS/CodKits_iOS.h>
#endif

NS_ASSUME_NONNULL_BEGIN

#pragma mark - Static String Keys
static NSString * kColoursRGBA_R    = @"RGBA-r";
static NSString * kColoursRGBA_G    = @"RGBA-g";
static NSString * kColoursRGBA_B    = @"RGBA-b";
static NSString * kColoursRGBA_A    = @"RGBA-a";
static NSString * kColoursHSBA_H    = @"HSBA-h";
static NSString * kColoursHSBA_S    = @"HSBA-s";
static NSString * kColoursHSBA_B    = @"HSBA-b";
static NSString * kColoursHSBA_A    = @"HSBA-a";
static NSString * kColoursCIE_L     = @"LABa-L";
static NSString * kColoursCIE_A     = @"LABa-A";
static NSString * kColoursCIE_B     = @"LABa-B";
static NSString * kColoursCIE_alpha = @"LABa-a";
static NSString * kColoursCIE_C     = @"LABa-C";
static NSString * kColoursCIE_H     = @"LABa-H";
static NSString * kColoursCMYK_C    = @"CMYK-c";
static NSString * kColoursCMYK_M    = @"CMYK-m";
static NSString * kColoursCMYK_Y    = @"CMYK-y";
static NSString * kColoursCMYK_K    = @"CMYK-k";

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSColor

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSColor (Colors)

+ (instancetype) colorWithR:(CGFloat) red G:(CGFloat) green B:(CGFloat) blue A:(CGFloat) alpha;

#pragma mark - Colors
// System Colors
+ (NSColor *) infoBlueColor;
+ (NSColor *) successColor;
+ (NSColor *) warningColor;
+ (NSColor *) dangerColor;

// Whites
+ (NSColor *) antiqueWhiteColor;
+ (NSColor *) oldLaceColor;
+ (NSColor *) ivoryColor;
+ (NSColor *) seashellColor;
+ (NSColor *) ghostWhiteColor;
+ (NSColor *) snowColor;
+ (NSColor *) linenColor;

// Grays
+ (NSColor *) black25PercentColor;
+ (NSColor *) black50PercentColor;
+ (NSColor *) black75PercentColor;
+ (NSColor *) warmGrayColor;
+ (NSColor *) coolGrayColor;
+ (NSColor *) charcoalColor;

// Blues
+ (NSColor *) tealColor;
+ (NSColor *) steelBlueColor;
+ (NSColor *) robinEggColor;
+ (NSColor *) pastelBlueColor;
+ (NSColor *) turquoiseColor;
+ (NSColor *) skyBlueColor;
+ (NSColor *) indigoColor;
+ (NSColor *) denimColor;
+ (NSColor *) blueberryColor;
+ (NSColor *) cornflowerColor;
+ (NSColor *) babyBlueColor;
+ (NSColor *) midnightBlueColor;
+ (NSColor *) fadedBlueColor;
+ (NSColor *) icebergColor;
+ (NSColor *) waveColor;

// Greens
+ (NSColor *) emeraldColor;
+ (NSColor *) grassColor;
+ (NSColor *) pastelGreenColor;
+ (NSColor *) seafoamColor;
+ (NSColor *) paleGreenColor;
+ (NSColor *) cactusGreenColor;
+ (NSColor *) chartreuseColor;
+ (NSColor *) hollyGreenColor;
+ (NSColor *) oliveColor;
+ (NSColor *) oliveDrabColor;
+ (NSColor *) moneyGreenColor;
+ (NSColor *) honeydewColor;
+ (NSColor *) limeColor;
+ (NSColor *) cardTableColor;

// Reds
+ (NSColor *) salmonColor;
+ (NSColor *) brickRedColor;
+ (NSColor *) easterPinkColor;
+ (NSColor *) grapefruitColor;
+ (NSColor *) pinkColor;
+ (NSColor *) indianRedColor;
+ (NSColor *) strawberryColor;
+ (NSColor *) coralColor;
+ (NSColor *) maroonColor;
+ (NSColor *) watermelonColor;
+ (NSColor *) tomatoColor;
+ (NSColor *) pinkLipstickColor;
+ (NSColor *) paleRoseColor;
+ (NSColor *) crimsonColor;

// Purples
+ (NSColor *) eggplantColor;
+ (NSColor *) pastelPurpleColor;
+ (NSColor *) palePurpleColor;
+ (NSColor *) coolPurpleColor;
+ (NSColor *) violetColor;
+ (NSColor *) plumColor;
+ (NSColor *) lavenderColor;
+ (NSColor *) raspberryColor;
+ (NSColor *) fuschiaColor;
+ (NSColor *) grapeColor;
+ (NSColor *) periwinkleColor;
+ (NSColor *) orchidColor;

// Yellows
+ (NSColor *) goldenrodColor;
+ (NSColor *) yellowGreenColor;
+ (NSColor *) bananaColor;
+ (NSColor *) mustardColor;
+ (NSColor *) buttermilkColor;
+ (NSColor *) goldColor;
+ (NSColor *) creamColor;
+ (NSColor *) lightCreamColor;
+ (NSColor *) wheatColor;
+ (NSColor *) beigeColor;

// Oranges
+ (NSColor *) peachColor;
+ (NSColor *) burntOrangeColor;
+ (NSColor *) pastelOrangeColor;
+ (NSColor *) cantaloupeColor;
+ (NSColor *) carrotColor;
+ (NSColor *) mandarinColor;

// Browns
+ (NSColor *) chiliPowderColor;
+ (NSColor *) burntSiennaColor;
+ (NSColor *) chocolateColor;
+ (NSColor *) coffeeColor;
+ (NSColor *) cinnamonColor;
+ (NSColor *) almondColor;
+ (NSColor *) eggshellColor;
+ (NSColor *) sandColor;
+ (NSColor *) mudColor;
+ (NSColor *) siennaColor;
+ (NSColor *) dustColor;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSColor (Hex)

- (NSString *_Nullable) hexRRGGBBString;
- (NSString *_Nullable) hexBBGGRRString;
- (NSString *_Nullable) hexAARRGGBBString;
- (NSString *_Nullable) hexAABBGGRRString;

+ (NSColor *_Nullable)  colorFromHexString:(NSString *_Nullable) hexValue;
+ (NSColor *_Nullable)  colorFromHexString:(NSString *_Nullable) hexValue withAlpha:(CGFloat) alpha;
+ (NSColor *_Nullable)  colorFromHexValue:(NSInteger) hexValue withAlpha:(CGFloat) alpha;
+ (NSColor *_Nullable)  colorFromHexValue:(NSInteger) hexValue;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSColor (Components)

#if TARGET_OS_IOS
@property(nonatomic, readonly) CGFloat redComponent;
@property(nonatomic, readonly) CGFloat greenComponent;
@property(nonatomic, readonly) CGFloat blueComponent;
@property(nonatomic, readonly) CGFloat alphaComponent;
#endif
@property(nonatomic, readonly) CGFloat blackComponent;
@property(nonatomic, readonly) CGFloat cyanComponent;
@property(nonatomic, readonly) CGFloat magentaComponent;
@property(nonatomic, readonly) CGFloat yellowComponent;

- (NSColor *_Nullable)  addBlueComponent:(float) value;
- (NSColor *_Nullable)  addGreenComponent:(float) value;
- (NSColor *_Nullable)  addRedComponent:(float) value;
- (NSColor *_Nullable)  addColor:(NSColor *_Nullable) add blendMode:(CGBlendMode) blendMode;
- (NSColor *_Nullable)  withAlpha:(CGFloat) withAlpha;

+ (NSColor *_Nullable)  colorWithCyan:(CGFloat) cyan magenta:(CGFloat) magenta yellow:(CGFloat) yellow black:(CGFloat) black alpha:(CGFloat) alpha;
+ (NSColor *_Nullable)  colorWithHue:(CGFloat) hue saturation:(CGFloat) saturation lightness:(CGFloat) lightness alpha:(CGFloat) alpha;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSColor (Convert)

+ (void)      RGB2HSL:(CGFloat) r g:(CGFloat) g b:(CGFloat) b h:(CGFloat *_Nullable) h s:(CGFloat *_Nullable) s l:(CGFloat *_Nullable) l;
+ (void)      HSL2RGB:(CGFloat) h s:(CGFloat) s l:(CGFloat) l r:(CGFloat *_Nullable) r g:(CGFloat *_Nullable) g b:(CGFloat *_Nullable) b;
+ (void)      HSL2HSB:(CGFloat) h s:(CGFloat) s l:(CGFloat) l hh:(CGFloat *_Nullable) hh ss:(CGFloat *_Nullable) ss bb:(CGFloat *_Nullable) bb;
+ (void)      RGB2HSB:(CGFloat) r g:(CGFloat) g b:(CGFloat) b h:(CGFloat *_Nullable) h s:(CGFloat *_Nullable) s v:(CGFloat *_Nullable) v;
+ (void)      HSB2RGB:(CGFloat) h s:(CGFloat) s v:(CGFloat) v r:(CGFloat *_Nullable) r g:(CGFloat *_Nullable) g b:(CGFloat *_Nullable) b;
+ (void)      RGB2CMYK:(CGFloat) r g:(CGFloat) g b:(CGFloat) b c:(CGFloat *_Nullable) c m:(CGFloat *_Nullable) m y:(CGFloat *_Nullable) y k:(CGFloat *_Nullable) k;
+ (void)      CMYK2RGB:(CGFloat) c m:(CGFloat) m y:(CGFloat) y k:(CGFloat) k r:(CGFloat *_Nullable) r g:(CGFloat *_Nullable) g b:(CGFloat *_Nullable) b;
+ (void)      HSB2HSL:(CGFloat) h s:(CGFloat) s b:(CGFloat) b hh:(CGFloat *_Nullable) hh ss:(CGFloat *_Nullable) ss ll:(CGFloat *_Nullable) ll;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSColor (Adjust)

- (NSColor *) lightenByValue:(float) value;
- (NSColor *) darkenByValue:(float) value;
- (BOOL)      isLightColor;
- (NSColor *) inverseColor;
- (NSColor *) modifyTintByPercentage:(CGFloat) percentage;
- (NSColor *) darkenByPercentage:(CGFloat) percentage;
- (NSColor *) lightenByPercentage:(CGFloat) percentage;
- (NSArray *) rgbaArray;
+ (NSColor *) colorFromRGBAArray:(NSArray *) rgbaArray;
+ (NSColor *) colorFromRGBADictionary:(NSDictionary *) rgbaDict;
+ (NSColor *) colorFromCMYKArray:(NSArray *) cmyk;
+ (NSColor *) colorFromCMYKDictionary:(NSDictionary *) cmyk;
+ (NSColor *) colorBetween2Colors:(NSColor *) fromColor toColor:(NSColor *) toColor withPercent:(CGFloat) withPercent;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSColor (Cie)

+ (NSColor *) colorFromCIE_LabArray:(NSArray *) colors;

/**
 Creates a Color from a dictionary of 4 NSNumbers
 Keys: kColoursCIE_L, kColoursCIE_A, kColoursCIE_B, kColoursCIE_alpha
 @param colors   4 NSNumbers for CIE_LAB between 0 - 1
 @return Color
 */
+ (NSColor *) colorFromCIE_LabDictionary:(NSDictionary *) colors;

/**
 Creates a Color from an array of 4 NSNumbers (L,a,b,alpha)
 @param colors   4 NSNumbers for CIE_LAB between 0 - 1
 @return Color
 */
+ (NSColor *) colorFromCIE_LCHArray:(NSArray *) colors;

/**
 Creates a Color from a dictionary of 4 NSNumbers
 Keys: kColoursCIE_L, kColoursCIE_A, kColoursCIE_B, kColoursCIE_alpha
 @param colors   4 NSNumbers for CIE_LAB between 0 - 1
 @return Color
 */
+ (NSColor *) colorFromCIE_LCHDictionary:(NSDictionary *) colors;

@end

NS_ASSUME_NONNULL_END

#endif


