#ifndef CKCoreNSMenu_h
#define CKCoreNSMenu_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

NS_ASSUME_NONNULL_BEGIN

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSMenu

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSMenu (Pretty)

// Create a new menu with a series of parsed menu items and/or submenus.
// Menu items that aren't a separator must be followed by a selector (or NULL).
// Submenus' titles are used for their respective items' titles.
// Preceed a menu item with PRETTY_ALTERNATE to make it an alternate.
+ (NSMenu *_Nullable) menuWithTitle:(NSString *_Nullable) title items:(id _Nullable) firstArg, ...
    NS_REQUIRES_NIL_TERMINATION;

- (NSMenuItem *) addMenuItemWithString:(NSString *) string;
- (NSMenuItem *) addMenuItemWithString:(NSString *) string action:(SEL _Nullable) action;
- (NSMenuItem *) addMenuItemWithString:(NSString *) string action:(SEL _Nullable) action isAlternate:(BOOL) isAlternate;
- (NSMenuItem *) addItemWithTitle:(NSString *) string action:(nullable SEL) selector keyEquivalent:(NSString *_Nullable) charCode state:(NSControlStateValue) state;
- (NSMenuItem *) addItemWithTitle:(NSString *) string action:(nullable SEL) selector keyEquivalent:(NSString *_Nullable) charCode state:(NSControlStateValue) state enabled:(Boolean) enabled;
- (NSMenuItem *) addItemWithTitle:(NSString *) string action:(nullable SEL) selector keyEquivalent:(NSString *_Nullable) charCode enabled:(Boolean) enabled;
- (NSMenuItem *) addItemWithTitle:(NSString *) string image:(NSImage *_Nullable) image action:(nullable SEL) selector keyEquivalent:(NSString *_Nullable) charCode enabled:(Boolean) enabled;
- (NSMenuItem *) addItemWithTitle:(NSString *) string image:(NSImage *) image action:(nullable SEL) selector keyEquivalent:(NSString *) charCode enabled:(Boolean) enabled tag:(NSInteger) tag;
- (NSMenuItem *) addItemWithTitle:(NSString *) string action:(nullable SEL) selector keyEquivalent:(NSString *) charCode enabled:(Boolean) enabled tag:(NSInteger) tag;
- (NSMenuItem *) addItemWithColor:(NSColor *) color action:(nullable SEL) selector keyEquivalent:(NSString *_Nullable) charCode enabled:(Boolean) enabled;
- (NSMenuItem *) addItemWithColor:(NSColor *) color action:(nullable SEL) selector keyEquivalent:(NSString *_Nullable) charCode state:(NSControlStateValue) state enabled:(Boolean) enabled;
- (void)         addSeparator;

// Create a new menu item by parsing the given string, then adds a new 
// submenu to it.
//
// Returns the newly-created submenu, or nil if parsing fails.
- (NSMenu *_Nullable) addSubmenuWithString:(NSString *_Nullable) string;
- (NSMenu *_Nullable) addSubmenuWithString:(NSString *) string image:(NSImage *) image;

// Add a menu as a submenu in an item with the same title.
- (void) addSubmenu:(NSMenu *_Nullable) menu;

extern NSString * _Nullable const PRETTY_ALTERNATE;
extern NSString * _Nullable const PRETTY_DISABLED;
extern NSString * _Nullable const PRETTY_ENABLED;

@end

NS_ASSUME_NONNULL_END

#endif

