#ifndef CKCoreOpenAI_h
#define CKCoreOpenAI_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

typedef void (^CKCoreOpenAIStreamCallback) (NSURLSessionDataTask *task, NSData *data, NSError *error);
typedef void (^CKCoreOpenAIChatCallback)   (NSDictionary *reply, NSError *error);

#define OPENAIMODEL_gpt_4o_mini    @"gpt-4o-mini"
#define OPENAIMODEL_gpt_4o         @"gpt-4o"
#define OPENAIMODEL_gpt_4_turbo    @"gpt-4-turbo"
#define OPENAIMODEL_gpt_4          @"gpt-4"
#define OPENAIMODEL_gpt_3_5_turbo  @"gpt-3.5-turbo"
#define OPENAIMODEL_gpt_3_5        @"gpt-3.5"

//----------------------------------------------------------------------------------------------------
@interface CKCoreOpenAI: NSObject

+ (void) promptTextCompletion:(NSString *) apiKey withPrompt:(NSString *) withPrompt completionBlock:(void (^)(NSString *result, NSError *error)) completionBlock;
+ (void) promptTextCompletion:(NSString *) apiKey withModel:(NSString *) withModel withTemperature:(CGFloat) withTemperature withPrompt:(NSString *) withPrompt completionBlock:(void (^)(NSString *result, NSError *error)) completionBlock;
+ (void) promptTextCompletion:(NSString *) apiKey withModel:(NSString *) withModel withTemperature:(CGFloat) withTemperature withMaxTokens:(NSInteger) withMaxTokens withPrompt:(NSString *) withPrompt completionBlock:(void (^)(NSString *result, NSError *error)) completionBlock;

+ (void) promptChatCompletion:(NSString *) apiKey withModel:(NSString *) withModel withTemperature:(CGFloat) withTemperature withMaxTokens:(NSInteger) withMaxTokens withPrompt:(NSString *) withPrompt completionBlock:(void (^)(NSString *result, NSDictionary *rawResponse, NSError *error)) completionBlock;

- (void) promptChatCompletionStream:(NSString *) apiKey withModel:(NSString *) withModel withTemperature:(CGFloat) withTemperature withMaxTokens:(NSInteger) withMaxTokens withPrompt:(NSString *) withPrompt completionBlock:(CKCoreOpenAIStreamCallback) completionBlock;
- (void) promptChatCompletion:(NSString *) apiKey withModel:(NSString *) withModel withTemperature:(CGFloat) withTemperature withMaxTokens:(NSInteger) withMaxTokens withPrompt:(NSString *) withPrompt completionBlock:(CKCoreOpenAIStreamCallback) completionBlock;

@end

//----------------------------------------------------------------------------------------------------
@interface CKCoreOpenAIChat: NSObject

- (void) promptChatCompletion:(NSString *) apiKey withModel:(NSString *) withModel withTemperature:(CGFloat) withTemperature withMaxTokens:(NSInteger) withMaxTokens withSystemRolePrompt:(NSString *) withSystemRolePrompt withUserRolePrompt:(NSString *) withUserRolePrompt completionBlock:(CKCoreOpenAIChatCallback) completionBlock;
- (void) promptChatCompletion:(NSString *) apiKey withModel:(NSString *) withModel withTemperature:(CGFloat) withTemperature withMaxTokens:(NSInteger) withMaxTokens withRolePrompts:(NSArray <NSDictionary *> *) withRolePrompts completionBlock:(CKCoreOpenAIChatCallback) completionBlock;

@end

#endif


