#ifndef CKCorePasswordStrengthEvaluator_h
#define CKCorePasswordStrengthEvaluator_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

//-------------------------------------------------------------------------------------------------------------------------
typedef NS_ENUM(NSUInteger, CKCorePasswordStrength)
  {
  CKCoreVeryWeakPasswordStrength,
  CKCoreWeakPasswordStrength,
  CKCoreReasonablePasswordStrength,
  CKCoreStrongPasswordStrength,
  CKCoreVeryStrongPasswordStrength,
  };

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCorePasswordStrengthEvaluator : NSObject

+ (CKCorePasswordStrength) strengthOfPassword:(NSString *) password;
+ (NSString *)             localizedStringForPasswordStrength:(CKCorePasswordStrength) strength;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCorePasswordValidator : NSObject

+ (instancetype) standardValidator;
+ (instancetype) validatorWithRules:(NSArray *) rules;

- (BOOL) validatePassword:(NSString *) password failingRules:(out NSArray * __autoreleasing *) rules;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
@protocol CKCorePasswordRule <NSObject>

- (BOOL)       evaluateWithString:(NSString *) string;
- (NSString *) localizedErrorDescription;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreAllowedCharacterRule : NSObject <CKCorePasswordRule>

+ (instancetype) ruleWithAllowedCharacters:(NSCharacterSet *) characterSet;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreRequiredCharacterRule : NSObject <CKCorePasswordRule>

+ (instancetype) ruleWithRequiredCharacters:(NSCharacterSet *) characterSet;
+ (instancetype) lowercaseCharacterRequiredRule;
+ (instancetype) uppercaseCharacterRequiredRule;
+ (instancetype) decimalDigitCharacterRequiredRule;
+ (instancetype) symbolCharacterRequiredRule;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreDictionaryWordRule : NSObject <CKCorePasswordRule>

+ (instancetype) rule;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreLengthRule : NSObject <CKCorePasswordRule>

+ (instancetype) ruleWithRange:(NSRange)range;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCorePredicateRule : NSObject <CKCorePasswordRule>

+ (instancetype) ruleWithPredicate:(NSPredicate *) predicate;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreRegularExpressionRule : NSObject <CKCorePasswordRule>

+ (instancetype) ruleWithRegularExpression:(NSRegularExpression *) regularExpression;

@end

#pragma mark -
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreBlockRule : NSObject <CKCorePasswordRule>

+ (instancetype) ruleWithBlock:(BOOL (^)(NSString *password)) block;

@end

extern CGFloat CKCoreEntropyForString(NSString *string);

#endif

