#ifndef CKCoreSystemInfo_h
#define CKCoreSystemInfo_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #import <CodKits_macOS/CodKits_macOS.h>
#else
  #import <CodKits_iOS/CodKits_iOS.h>
#endif

#define SYSTEM_NATIVE_EXECUTION  0
#define SYSTEM_ROSETTA_EXECUTION 1

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo: NSObject


@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (APPRun)

+ (Boolean) isDebuggerAttached;
+ (Boolean) isCommandLineApplication;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (Device)

+ (NSString *) deviceSerialNumber;
+ (NSUInteger) numberOfProcessors;
+ (NSInteger)  numberOfActiveThreads;
+ (NSUInteger) numberOfActiveProcessors;
+ (NSInteger)  processExecutionMode;
+ (Boolean)    killApplicationByBundleID:(NSString *) withBundleID;
+ (Boolean)    forceKillApplicationByBundleID:(NSString *) withBundleID;
+ (NSArray *)  getListOfProcessesByName:(NSString *) withAppName;
+ (uint64_t)   freeDiskspace;
+ (NSString *) diskID;
+ (CGFloat)    GbRAMSize;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (APPVersion)

+ (NSString *) appVersionShort;
+ (NSString *) appVersionLong;
+ (NSString *) appVersion;
+ (NSString *) appVersionBuild;
+ (NSString *) appBundleIdentifier;
+ (NSString *) appBundleName;
+ (Boolean)    isAPPRunningInXCode;
+ (void)       appStoreInfo:(void (^)(NSDictionary * dictionary)) completionHandler;
+ (void)       isAppStoreUpdateAvailable:(void (^)(Boolean updateAvailable, NSString *newVersion, NSDictionary *dictionary)) completionHandler;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (GPU)

+ (NSArray *) gpuDevicesList;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemInfo (Screen)

+ (CGFloat)        screenWidth;
+ (CGFloat)        screenRealWidth;
+ (CGFloat)        screenHeight;
+ (CGFloat)        screenRealHeight;
+ (CGFloat)        screenRatio;
+ (CGSize)         screenSize;
+ (CGSize)         screenRealSize;
+ (CGRect)         screenBounds;
+ (CGFloat)        screenScale;
+ (CGFloat)        screenPPi;
+ (CGFloat)        screenHorizontalInches;
+ (CGFloat)        screenVerticalInches;
+ (CGFloat)        screenDiagonalInches;
+ (NSDictionary *) screenDimensions;
+ (Boolean)        screenInDarkMode;
+ (NSWindow *)     screenKeyWindow;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreSystemSleep: NSObject

- (Boolean) preventDisplaySleep:(NSString *) withReason;
- (Boolean) preventComputerSleep:(NSString *) withReason;

@end

#endif 
