#ifndef CKCoreNSAttributedString_h
#define CKCoreNSAttributedString_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

//NSAttributedString *blue, *green, *never, *result;
//
//blue = [[NSAttributedString alloc] initWithString:@"Blue" attributes:@{
//    NSFontAttributeName: [UIFont boldSystemFontOfSize:16.0f],
//    NSForegroundColorAttributeName: [UIColor blueColor],
//}];
//
//green = [[NSAttributedString alloc] initWithString:@"Green" attributes:@{
//    NSFontAttributeName: [UIFont boldSystemFontOfSize:16.0f],
//    NSForegroundColorAttributeName: [UIColor greenColor],
//}];
//
//never = [[NSAttributedString alloc] initWithString:@"never" attributes:@{
//    NSStrikethroughStyleAttributeName: @(NSUnderlineStyleSingle),
//}];
//
//result = [NSAttributedString attributedStringWithFormat:@"%@ and %@ must %@ be seen", blue, green, never];

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSAttributedString

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
/** Attributed string extension for creating attributed strings from a format string. */
@interface NSAttributedString (Build)

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param format A format string. This value must not be nil.
 @param ... A comma-separated list of arguments to substitute into format.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
+ (instancetype) attributedStringWithFormat:(NSString *) format, ... NS_FORMAT_FUNCTION(1,2);

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param attributes A dictionary of attributes for the format string. May be nil.
 @param format A format string. This value must not be nil.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
+ (instancetype) attributedStringWithAttributes:(NSDictionary *) attributes format:(NSString *) format, ... NS_FORMAT_FUNCTION(2,3);

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param format A format string. This value must not be nil.
 @param arguments list of arguments to substitute into format.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
+ (instancetype) attributedStringWithFormat:(NSString *) format arguments:(va_list) arguments NS_FORMAT_FUNCTION(1,0);

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param attributes A dictionary of attributes for the format string. May be nil.
 @param format A format string. This value must not be nil.
 @param arguments list of arguments to substitute into format.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
+ (instancetype) attributedStringWithAttributes:(NSDictionary *) attributes format:(NSString *) format arguments:(va_list) arguments NS_FORMAT_FUNCTION(2,0);

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param format A format string. This value must not be nil.
 @param ... A comma-separated list of arguments to substitute into format.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
- (instancetype) initWithFormat:(NSString *) format, ... NS_FORMAT_FUNCTION(1,2);

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param attributes A dictionary of attributes for the format string. May be nil.
 @param format A format string. This value must not be nil.
 @param ... A comma-separated list of arguments to substitute into format.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
- (instancetype) initWithAttributes:(NSDictionary *) attributes format:(NSString *) format, ... NS_FORMAT_FUNCTION(2,3);

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param format A format string. This value must not be nil.
 @param arguments list of arguments to substitute into format.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
- (instancetype) initWithFormat:(NSString *) format arguments:(va_list) arguments NS_FORMAT_FUNCTION(1,0);

/** Returns an attributed string created by using a given format string as a template into which the remaining argument values are substituted.
 @param attributes A dictionary of attributes for the format string. May be nil.
 @param format A format string. This value must not be nil.
 @param arguments list of arguments to substitute into format.
 @return An attributed string created by using format as a template into which the remaining argument values are substituted.
 */
- (instancetype) initWithAttributes:(NSDictionary *) attributes format:(NSString *) format arguments:(va_list) arguments NS_FORMAT_FUNCTION(2,0);

@end

#if TARGET_OS_IOS
//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSAttributedString (HTML)

/**
 *  Generates an attributed string from HTML.
 *
 *  @param htmlString HTML String
 *
 *  @return Attributed string
 */
+ (NSAttributedString *) attributedStringFromHTML:(NSString *) htmlString;

/**
 *  Generates an attributed string from HTML.
 *
 *  @param htmlString HTML String
 *  @param boldFont   Font to use for \<b\> and \<strong\> tags
 *  @param italicFont Font to use for \<i\> and \<em\> tags
 *
 *  @return Attributed string
 */
+ (NSAttributedString *) attributedStringFromHTML:(NSString *) htmlString boldFont:(NSFont *) boldFont italicFont:(NSFont *) italicFont;

/**
 *  Generates an attributed string from HTML.
 *
 *  @param htmlString HTML String
 *  @param normalFont Font to use for general text
 *  @param boldFont   Font to use for \<b\> and \<strong\> tags
 *  @param italicFont Font to use for \<i\> and \<em\> tags
 *
 *  @return Attributed string
 */
+ (NSAttributedString *) attributedStringFromHTML:(NSString *) htmlString normalFont:(NSFont *) normalFont boldFont:(NSFont *) boldFont italicFont:(NSFont *) italicFont;

/**
 *  Generates an attributed string from HTML.
 *
 *  @param htmlString   HTML String
 *  @param normalFont   Font to use for general text
 *  @param boldFont     Font to use for \<b\> and \<strong\> tags
 *  @param italicFont   Font to use for \<i\> and \<em\> tags
 *  @param imageMap     Images to use in place of standard bundle images.
 *
 *  @return Attributed string
 */
+ (NSAttributedString *) attributedStringFromHTML:(NSString *) htmlString normalFont:(NSFont *) normalFont boldFont:(NSFont *) boldFont italicFont:(NSFont *) italicFont imageMap:(NSDictionary<NSString *, NSImage *> *) imageMap;

@end
#endif

#endif

