#ifndef CKCoreNSImage_h
#define CKCoreNSImage_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <AVFoundation/AVFoundation.h>
#import <ImageIO/ImageIO.h>

#import <AVFoundation/AVFoundation.h>

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNSImage

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Async)

+ (void) imageFromUrl:(NSURL *) withUrl completionBlock:(void (^)(NSImage *image)) completionBlock;
+ (void) imageFromUrlWithCache:(NSURL *) withImageUrl withCacheDirectory:(NSURL *) withCacheDirectory completionBlock:(void (^)(NSImage *image)) completionBlock;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (CGImage)

- (CGImageRef) CGImage;
+ (NSImage *)  imageWithCGImage:(CGImageRef) imageRef;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Mask)

+ (NSImage *) imageWithMaskImage:(NSImage *) image maskImage:(NSImage *) maskImage;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (CIImage)

- (CIImage *) CIImage;

+ (NSImage *) imageWithCIImage:(CIImage *) imageRef;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (FillColor)

+ (NSImage *) filledWithColor:(CGSize) withSize withColor:(NSColor *) withColor;
- (NSImage *) tintWithColor:(NSColor *) withColor;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Scale)

- (NSImage *) scaleToPercentSize:(CGFloat) withPercent;
- (NSImage *) scaleToSize:(CGSize) withSize;
- (NSImage *) scaleToHeight:(NSInteger) withHeight;
- (NSImage *) scaleToWidth:(NSInteger) withWidth;
- (NSImage *) scaleToFitSize:(CGSize) withSize;
- (NSImage *) scaleToFillSize:(CGSize) withSize;
- (NSImage *) scaleToPixelDimensions:(NSSize) newSize;
- (Boolean)   isLandscape;

+ (NSImage *) resizedImage:(NSImage *) sourceImage toPixelDimensions:(NSSize) newSize;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (EncodeQRCode)

+ (NSImage *) generateQRCodeFromString:(NSString *) QRString withSize:(CGFloat) Size;
+ (NSImage *) generateQRCodeFromString:(NSString *) QRString withSize:(CGFloat) ImageSize withFillColor:(NSColor *) FillColor;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Extended)

- (NSData *)  PNGRepresentation:(CGFloat) withFactor;
- (NSData *)  JPGRepresentation:(CGFloat) withFactor;
- (Boolean)   saveAsPNG:(NSString *) toFilePath withFactor:(CGFloat) withFactor;
- (Boolean)   saveAsJPG:(NSString *) toFilePath withFactor:(CGFloat) withFactor;
- (NSImage *) cropToRect:(CGRect) withSourceRect;
- (NSImage *) cropToCircle:(CGRect) withCroppedRect;

+ (NSImage *) createFromRGBAFloatArray:(NSInteger) imageWidth imageHeight:(NSInteger) imageHeight imageData:(float *) imageData;
+ (NSImage *) createNSImageFromRGBA:(unsigned char *) rgbaBytes width:(NSInteger) width height:(NSInteger) height;
+ (NSImage *) imageWithContentsOfURL:(NSURL *) url;
+ (NSImage *) imageWithData:(NSData *) imageData;
+ (NSImage *) imageWithContentsOfFile:(NSString *) path;

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface NSImage (Text)

- (NSImage *) imageWithCenteredText:(NSString *) text withfont:(NSFont *) font;
- (NSImage *) imageWithTopRightText:(NSString *) text withfont:(NSFont *) font withOffset:(CGFloat) withOffset;
- (NSImage *) imageWithText:(NSString *) text atPoint:(CGPoint) point withFont:(NSFont *) font;
- (NSImage *) imageWithText:(NSString *) text atPoint:(CGPoint) point withFont:(NSFont *) font withAlignment:(NSTextAlignment) withAlignment withColor:(NSColor *) withColor;

@end

#endif

