#ifndef CKCoreNetwork_h
#define CKCoreNetwork_h

#import <Foundation/Foundation.h>

#if TARGET_OS_OSX
  #include <CodKits_macOS/CodKits_macOS.h>
  #import <Foundation/Foundation.h>
  #import <SystemConfiguration/SystemConfiguration.h>
  #import <netinet/in.h>
  #import <CoreTelephony/CTTelephonyNetworkInfo.h>
#else
  #include <CodKits_iOS/CodKits_iOS.h>
#endif

#import <CoreWLAN/CoreWLAN.h>

NS_ASSUME_NONNULL_BEGIN

#define CKCoreEVT_NetworkStatusChanged                    @"CKCoreEVT_NetworkStatusChanged"

//-------------------------------------------------------------------------------------------------------------------------
// 3G         DN:0.5Mbps  UP:0.05Mbps
// 4G 3 bars  DN:1.24Mbps UP:0.14Mbps
// LTE 2 Bars DN:1.44Mbps UP:1.49Mbps
// WIFI       DN:60Mbps   UP:24Mbps
//-------------------------------------------------------------------------------------------------------------------------
typedef NS_ENUM(NSInteger, CKCoreNetworkStatus)
  {
  CKCoreNetworkNotReachable = 0,
  CKCoreNetworkReachableViaWiFi,
  CKCoreNetworkReachableViaWWAN
  };

typedef NS_ENUM(NSInteger, CKCoreNetworkConnectivityType)
  {
  CKCoreNetworkUnknown = 0,
  CKCoreNetworkGPRS,
  CKCoreNetworkEdge,
  CKCoreNetworkWCDMA,
  CKCoreNetworkHSDPA,
  CKCoreNetworkHSUPA,
  CKCoreNetworkCDMA1x,
  CKCoreNetworkCDMAEVDORev0,
  CKCoreNetworkCDMAEVDORevA,
  CKCoreNetworkCDMAEVDORevB,
  CKCoreNetworkeHRPD,
  CKCoreNetworkLTE
  };

extern NSString *kReachabilityChangedNotification;

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNetwork

@end

//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNetworkWatch: NSObject

//----------------------------------------------------------------------------------------------------
@property (nonatomic, readwrite) CKCoreNetworkStatus           netStatus;
@property (nonatomic, readwrite) CKCoreNetworkConnectivityType netType;
@property (nonatomic, readwrite) Boolean                       isNetworkConnected;

//----------------------------------------------------------------------------------------------------
+ (CKCoreNetworkWatch *_Nullable) sharedInstance;

- (void)                          setAPPBecameActive;
- (void)                          setAPPResignedActive;

@end


//-------------------------------------------------------------------------------------------------------------------------
// INTERFACE
//-------------------------------------------------------------------------------------------------------------------------
@interface CKCoreNetworkInfo: NSObject

+ (instancetype) reachabilityWithHostName:(NSString *) hostName;
+ (instancetype) reachabilityWithAddress:(const struct sockaddr *) hostAddress;
+ (instancetype) reachabilityForInternetConnection;

- (BOOL)                              startNotifier;
- (void)                              stopNotifier;
- (CKCoreNetworkStatus)               currentReachabilityStatus;
- (BOOL)                              connectionRequired;

+ (void)                              listOfAvailableWifiAPs:(void (^)(NSArray<CWNetwork *> *listOfAPs)) completionBlock;

@end

NS_ASSUME_NONNULL_END

#endif

